#jp-main-dock-panel {
  background-color: var(--jp-layout-color0);
}

/* .jp-waldiez-runButton {
  background-color: var(--md-green-500);
}
.jp-waldiez-runButton:hover {
  background-color: var(--md-green-700);
} */
